# frozen_string_literal: true

module Eost
  HT = %w[block_num block_time contract action from to amount symbol memo data].freeze
  R1 = '%<v5>-12.12s %<v6>-12.12s'
  R2 = '%<v7>10.5f %<v8>-8.8s'

  # trabalhar com folhas calculo bloks.io & dados no bigquery
  class Bigquery
    # processa folha calculo
    def processa_csv
      n = 0
      folha.sheet(0).parse(header_search: HT) do |r|
        n += 1
        puts n == 1 ? "\n#{folha.info}" : processa_row(r)
      end
    end

    # processa linha folha calculo para arquivo
    #
    # @param [Hash] has da linha em processamento
    # @return [String] texto informativo do processamento
    def processa_row(has)
      @row = has.values

      # array.count  = 0 ==> pode carregar esta linha
      # array.count >= 1 ==> nao carregar esta linha
      sql("select #{eos_fields} #{sql_where}", [{}, {}])

      if row_naoexiste? then row_str + (eos_insert_csv == 1 ? ' NOVA' : ' ERRO')
      elsif row_existe? then row_existente
      else                   row_multiplas
      end
    end

    # @return [String] linha folha calculo formatada
    def row_str
      "#{row[0]} #{Time.parse(row[1]).strftime(DF)} " + row_r1 + row_r2
    end

    # @return [String] linha folha calculo formatada
    def row_r1
      format(R1, v5: row[4], v6: row[5])
    end

    # @return [String] linha folha calculo formatada
    def row_r2
      format(R2, v7: Float(row[6]), v8: row[7])
    end

    # @return [String] linha folha calculo existente
    def row_existente
      d = linha[:e] ? dml("delete #{sql_where}") : 0
      "#{row_str} EXISTENTE#{str_apagadas(d)}"
    end

    # @return [String] linha folha calculo existencia multipla
    def row_multiplas
      d = linha[:m] ? dml("delete #{sql_where}") : 0
      "#{row_str} MULTIPLAS #{sql.count}#{str_apagadas(d)}"
    end

    # @param [Integer] num numero linhas apagadas
    # @return [String] texto formatado linhas apagadas
    def str_apagadas(num)
      num.positive? ? " & #{num} APAGADA(S) " : ' '
    end

    # @return [Boolean] linha folha calculo nao existe no bigquery?
    def row_naoexiste?
      sqr.count.zero?
    end

    # @return [Boolean] linha folha calculo existe no bigquery?
    def row_existe?
      sqr.count == 1
    end
  end
end
